REBOL [
	Title:	 "Graphics Functions"
	Author:  "Carl Sassenrath"
	File:	 %gfx-funcs.r
	Rights: "Copyright 2000 REBOL Technologies. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

screen-offset?: func [
	"Returns the absolute screen offset for any face."
	face [object!]
	/local xy
][
	xy: face/offset
	while [face: face/parent-face] [
		xy: xy + face/offset
		if face/edge [xy: xy + face/edge/size]
	]
	xy
]

win-offset?: func [
	"Returns the offset of a face within its window."
	face [object!]
	/local xy
][
	xy: 0x0
	while [face/parent-face] [
		xy: xy + face/offset
		face: face/parent-face
		if face/edge [xy: xy + face/edge/size]
	]
	xy
]

span?: func [
	"Returns a block of [min max] bounds for all faces"
	pane [block!]
	/part count [integer!] "Limit the number of faces"
	/local origin margin
][
	origin: 100000x100000
	margin: 0x0
	foreach item pane [
		if object? item [
			if all [count negative? count: count - 1][break]
			origin: min origin item/offset
			margin: max margin item/offset + item/size
		]
	]
	reduce [origin margin]
]

confine: func [
	"Return the correct offset to keep rectangular area in-bounds."
	offset [pair!] "Initial offset"
	size   [pair!] "Size of area"
	origin [pair!] "Lower bound (upper left)"
	margin [pair!] "Upper bound (lower right)"
][
	if offset/x < origin/x [offset/x: origin/x]
	if offset/y < origin/y [offset/y: origin/y]
	margin: margin - size
	if offset/x > margin/x [offset/x: margin/x]
	if offset/y > margin/y [offset/y: margin/y]
	offset
]

inside?: func [
	"TRUE if both X and Y of the second pair are less than the first."
	p1 [pair!]
	p2 [pair!]
][
	found? all [p2/x < p1/x p2/y < p1/y]
]

outside?: func [
	"TRUE if either X and Y of the second pair are greater than the first."
	p1 [pair!]
	p2 [pair!]
][
	found? any [p2/x > p1/x p2/y > p1/y]
]

within?: func[
	"Return TRUE if the point is within the rectangle bounds."
	point  [pair!] "XY position"
	offset [pair!] "Offset of area"
	size   [pair!] "Size of area"
][
	found? all [
		point/x >= offset/x
		point/y >= offset/y
		point/x < (offset/x + size/x)
		point/y < (offset/y + size/y)
	]
]

brightness?: func [
	"Returns the monochrome brightness (0.0 to 1.0) for a color value."
	color [tuple!]
][
	(.27 * color/1) + (.63 * color/2) + (.1 * color/3) / 255
]

overlap?: func [
 	"Returns TRUE if faces overlap each other."
	f1 f2
][
	found? all [
		f1/offset/x < (f2/offset/x + f2/size/x)
		f1/offset/y < (f2/offset/y + f2/size/y)
		(f1/offset/x + f1/size/x) > f2/offset/x
		(f1/offset/y + f1/size/y) > f2/offset/y
	]
]

edge-size?: func [
	"Return total size of face edge (both sides), even if missing edge."
	face [object!]
][
	either face/edge [face/edge/size * 2][0x0]
]
