REBOL [
	Title:  "PRebol - REBOL Preprocessor"
	Version: 2.7.8
	Rights: "Copyright REBOL Technologies 2011. All rights reserved."
	Home: http://www.rebol.com
	Date: 1-Jan-2011

	; You are free to use, modify, and distribute this file as long as the
	; above header, copyright, and this entire comment remains intact.
	; This software is provided "as is" without warranties of any kind.
	; In no event shall REBOL Technologies or source contributors be liable
	; for any damages of any kind, even if advised of the possibility of such
	; damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes. See http://www.rebol.com/support.html for details.
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

system/options/binary-base: 64  ; output data in base-64

verbose: off
size: 0 ; total size of all files (for stats)
where: "" ; file name, for error messages
incl-count: 0 ; number of include files

abort: func [msg] [print rejoin ["***ERROR (" where "): " reform msg] halt]

load-script: func [file [file! url!] keep /local data] [
	; Load a REBOL script file. /keep used to keep header of first script.
	; REBOL header is optional for all files.
	if verbose [print [tab "Code:" file]]
	if all [keep not script? file] [abort ["Missing script header:" file]]
	if error? try [data: load/all file] [abort ["Cannot load file:" file]]
	if all [not keep data/1 = 'rebol] [remove/part data 2]
	size: size + any [size? file 0]
	incl-count: incl-count + 1
	data
]

read-file: func [file [file! url!] /binary /local data] [
	; Read text or binary file.
	if verbose [print [tab pick ["Text:" "Data:"] not binary file]]
	if none? attempt [data: either binary [read/binary file][read file]] [
		abort ["Cannot read file:" file]
	]
	size: size + length? data
	incl-count: incl-count + 1
	data
]

check-file: func [file] [ ; args can be (expr) %file url
	; Check for valid filename, either direct or indirect thru expression.
	; Paren expression such as #include (join %file num: num + 1)
	if paren? :file [file: do-expr reduce [file]] ; for older REBOL compatibility
	if not any [file? file url? file] [abort ["Expected a file name:" mold file]]
	if not exists? file [abort ["File not found:" file]]
	file
]

do-expr: func [expr /local result] [
	; Evaluate expression and make sure it returns a result.
	either unset? set/any 'result do expr [none][:result]
]

process-source: func [
	; Process REBOL source code. (If a block, it's modified.) Returns block.
	; Also modifies these globals: size, where
	file [file! block! paren!] "Source to process"
	/keep "Keep rebol header"
	/local cmd commands path blk only new save-dir save-where
][
	; If a file was provided, load it:
	either any [file? file paren? file] [
		file: check-file file
		save-where: where
		where: file
		set [path file] split-path file
		save-dir: what-dir
		if path <> %./ [change-dir path]
		blk: load-script file keep
		if not block? blk [
			where: save-where
			change-dir save-dir
			return blk
		]
	][blk: file]

	while [not tail? blk][
		;-- Process all #commands:
		;   If a command returns a value, it is inserted into the output
		;   file. If it returns NONE, nothing is inserted. If you need
		;   to insert a NONE, return the word NONE, not the NONE value.
		either issue? cmd: first blk [
			only: none
			commands: [ ; #command size [action]
				; #include %file - insert REBOL code, data, loaded images, sounds.
				#include 2 [process-source blk/2]
				; #do [expression] - evaluate code, insert result.
				#do 2 [do-expr blk/2]
				; #if [condition] [then-expr] - conditional execution
				#if 3 [if do-expr blk/2 [blk/3]]
				; #either [condition] [then-expr] [else-expr]
				#either 4 [either do-expr blk/2 [blk/3][blk/4]]
				; #include-string %file - raw text data
				#include-string 2 [read-file check-file blk/2]
				; #include-binary %file - raw binary data (unloaded images, sounds).
				#include-binary 2 [read-file/binary check-file blk/2]
				; #include-files %
				#include-files 3 [ ; #include-files %path [file1 file2...]
					only: true
					new: copy []
					path: dirize blk/2
					if not file? path [abort ["Expected file path:" path]]
					foreach file blk/3 [
						append new file
						file: join path to-file file
						if not file? file [abort ["Expected file name:" file]]
						append/only new read-file/binary check-file file
					]
					new
				]
			]
			; Find, process and remove command, otherwise skip the #value.
			either cmd: find commands cmd [
				new: do cmd/3
				remove/part blk cmd/2
				either only [insert/only blk :new][if :new [insert blk :new]]
			][blk: next blk] ; a non-command #issue
		][
			if block? cmd [process-source cmd]
			blk: next blk
		] ; a non-issue value
	]

	if save-where [where: save-where]
	if save-dir [change-dir save-dir]
	head blk
]
