REBOL [
	Title:  "REBOL Protocols: IMAP"
	Author: "Holger Kruse"
	Rights: "Copyright REBOL Technologies 1997-2002"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

make Root-Protocol [
	{Communicate with IMAP.  This protocol is block oriented not
	string oriented.}
	
	port-flags: system/standard/port-flags/pass-thru
	close-check: ["Q1 LOGOUT" none]

	imap-parser: make object! [
		space: charset " ^-"
		text-char: complement space
		;string-char: complement union space charset {"\}
		; the following change fixes RAMBO #4147
		; also, i don't think the RFC states that quoted strings cannot contain spaces,
		; so I'd guess not allowing spaces here was not intended. -Gabriele
		string-char: complement charset {"\}
		atom-char: complement union space charset {^{"\()%*[]}
		flag-char: complement union space charset {^{"()[]}
		tag-char: complement union space charset {^{"\()%*+[]}
		int-char: charset [#"0" - #"9"]
		atom: [
			mark1: some atom-char
			["[" some atom-char "]" | none]
			mark2: (append last stack copy*/part mark1 mark2)
		]
		flag: [mark1: "\" some flag-char mark2: (append last stack copy*/part mark1 mark2)]
		tag: [mark1: some tag-char mark2: (append last stack copy*/part mark1 mark2)]
		string: [{"} mark1: any [string-char | ["\" skip]] mark2: {"} (
			tmp: copy*/part mark1 mark2
			forall tmp [if (first tmp) = #"\" [system/words/remove tmp]]
			append last stack head tmp
		)]
		literal: [mark1: "{" (
			tmp: load/next/all mark1
			append last stack first tmp
			mark1: skip mark1 ((length? mark1) - length? second tmp)
			) :mark1
		]
		text: [mark1: any text-char mark2: (append last stack copy*/part mark1 mark2)]
		integer: [mark1: some int-char mark2: (append last stack to-integer copy*/part mark1 mark2)]
		item-list: [
			[atom | flag | paren-set | string | literal] any [
				some space [atom | flag | paren-set | string | literal]
			]
		]
		list-contents: [
			any space (tmp: copy* [] append/only last stack tmp append/only stack tmp)
			[item-list | none]
			(system/words/remove back tail stack) any space
		]
		paren-set: [ "(" list-contents ")" ]
		resp-code: [ "[" list-contents "]" ]
		tagged-response: [
			[tag | "*" (append last stack '*)]
			some space
			[integer | atom]
			[[some space resp-code] | none (append last stack none)]
			[some space [item-list | text] | none (append last stack none)]
		]
		cont-response: [
			"+" (append last stack '+)
			some space
			[mark1: (append last stack copy* mark1) to end]
		]
		response: [[tagged-response | cont-response] any space]
		stack: result: mark1: mark2: tmp: none
		do-parse: func [str] [
			stack: copy* []
			result: copy* []
			append/only stack result
			if not parse/all str response [
				net-error "Parse error"
			]
			result
		]
	]

	imap-url-parser: make object! [
		mailbox-char: complement charset [#";" #"?"]
		integer-char: charset [#"0" - #"9"]
		mailbox: [
			mark1: any mailbox-char mark2:
			(object/name: copy*/part mark1 mark2)
		]
		integer: [some integer-char]
		search: ["?" mark1: (object/search: copy* mark1) to end]
		uidvalidity: [";UIDVALIDITY=" mark1: integer mark2:
			(object/uidvalidity: to-integer copy*/part mark1 mark2)
		]
		iuid: [["/" | none] ";UID=" mark1: integer mark2:
			(object/uid: to-integer copy*/part mark1 mark2
			if all [0 < length? object/name #"/" = last object/name] [
				system/words/remove back tail object/name
			])
		]
		isection: ["/;SECTION=" mark1: (object/section: copy* mark1) to end]
		list-input: [
			mailbox (object/type: 'list) ";TYPE=" mark1:
			["LIST" | "LSUB"] mark2: (object/list: uppercase copy*/part mark1 mark2)
		]
		search-input: [
			mailbox (object/type: 'search) [uidvalidity | none] search
		]
		uid-input: [
			mailbox (object/type: 'iuid) [uidvalidity | none] iuid [isection | none]
		]
		def-input: [
			mailbox (object/type: 'box) [uidvalidity | none]
		]
		input: [list-input | search-input | uid-input | def-input]
		object: none
		do-parse: func [str obj /local mark1 mark2] [
			object: obj
			if not parse/all str input [
				net-error "Invalid IMAP URL"
			]
		]
	]

	imap-read-literal: func [port count /local result tmp len] [
		result: make string! count
		while [count > 0] [
			len: read-io port tmp: make binary! count count
			if len <= 0 [
				net-error "Read error"
			]
			append result tmp
			count: count - len
		]
		result
	]

	imap-read-line: func [port /local server-said tmp ix count] [
		server-said: make string! 80
		forever [
			tmp: system/words/pick port 1
			if none? tmp [
				net-error "Server closed the connection"
			]
			either (last tmp) = #"}" [
				if not ix: find/last tmp #"{" [
					net-error "Malformed server response"
				]
				append server-said copy*/part tmp ix
				count: to-integer copy*/part next ix back tail tmp
				append server-said mold imap-read-literal port count
			] [
				append server-said tmp
				net-utils/net-log server-said
				return server-said
			]
		]
	]

	imap-transact: func [port send-data cont-block /local resp data w list-block tmp tmp2] [
		if send-data [
			port/locals/last-id: port/locals/last-id + 1
			resp: rejoin ["A" port/locals/last-id " " send-data]
			net-utils/net-log resp
			insert port/sub-port resp
		]
		forever [
			resp: imap-parser/do-parse imap-read-line port/sub-port
			if block? tmp: system/words/pick resp 3 [
				if find tmp "alert" [
					if not system/options/quiet [print ["IMAP Alert:" form at resp 4]]
				]
				if tmp2: find tmp "uidvalidity" [
					error? try [port/locals/uidvalidity: to-integer second tmp2]
				]
				if (first tmp) = "capability" [
					port/locals/capabilities: union port/locals/capabilities at tmp 2
				]
			]
			either (first resp) = '* [
				either integer? w: second resp [
					switch fourth resp [
						"exists" [
							if port/locals/access/type = 'box [
								port/state/tail: w
							]
						]
						"fetch" [
							if any [
							 all [port/locals/access/type = 'box w = port/locals/msg-num]
							 all [find [iuid search] port/locals/access/type
							  (select fifth resp "uid") = to-string port/locals/msg-num]] [
								port/locals/msg: select fifth resp port/locals/recv-section
								replace/all port/locals/msg "^M" ""
							]
						]
						"expunge" [
							port/state/tail: port/state/tail - 1
						]
					]
				] [
					switch w [
						"capability" [
							port/locals/capabilities: union port/locals/capabilities at resp 5
						]
						"list" "lsub" [
							if all [
								port/locals/hierarchy-delimiter: system/words/pick resp 5
								not empty? tmp: system/words/pick resp 6
							] [
								append port/locals/list tmp
								port/state/tail: port/state/tail + 1
							]
						]
						"search" [
							tmp: at resp 4
							forall tmp [
								all [first tmp append port/locals/msg-uids to-integer first tmp]
							]
						]
					]
				]
			] [
				either (first resp) = '+ [
					bind cont-block 'resp
					do cont-block
				] [
					if (first resp) = rejoin ["A" port/locals/last-id] [
						return resp
					]
				]
			]
			if not send-data [return resp]
		]
	]

	imap-check: func [port send-data cont-block expected /local resp] [
		resp: imap-transact port send-data cont-block
		if not find expected to-word second resp [
			net-error reform ["Server error: IMAP" resp]
		]
		return resp
	]

	imap-do-cram-md5: func [port server-data /local send-data] [
		server-data: debase/base second server-data 64
		send-data: reform [port/locals/user-name
		 lowercase enbase/base checksum/method/key server-data 'md5 port/pass 16]
		send-data: enbase/base send-data 64
		net-utils/net-log send-data
		insert port/sub-port send-data
	]

	imap-form-string: func [str /local res c] [
		res: make string! 2 + length? str
		append res #"^""
		foreach c str [
			if find {"\} c [append res #"\"]
			append res c
		]
		append res #"^""
		res
	]

	open: func [port /local resp reqcap auth-done auth-mode auth-modes path select-block][
		resp: parse/all port/user ";"
		port/locals: make object! [
			last-id: 0
			capabilities: copy* []
			msg-uids: send-section: recv-section: uidvalidity: list: msg-num: msg: none
			user-name: system/words/pick resp 1
			access: make object! [
				type: name: search: uidvalidity: uid: section: list: none
			]
			hierarchy-delimiter: none
		]
		open-proto port
		auth-mode: system/words/pick resp 2
		resp: imap-check port none none [ok preauth]
		auth-done: (second resp) = 'preauth
		reqcap: copy* []
		auth-modes: ["auth=login" "auth=cram-md5"]
		if not auth-done [
			append reqcap auth-modes
		]
		if not empty? exclude reqcap port/locals/capabilities [
			imap-check port "CAPABILITY" none [ok]
		]
		if all [not auth-done auth-mode (auth-mode <> "auth=*")] [
			port/locals/capabilities: exclude port/locals/capabilities exclude auth-modes auth-mode
		]
		if all [not auth-done find port/locals/capabilities "auth=cram-md5"] [
			if not error? catch [
				imap-check port "AUTHENTICATE CRAM-MD5" [imap-do-cram-md5 port resp] [ok]
			] [
				auth-done: true
			]
		]
		if not auth-done [
			if not error? catch [
				imap-check port reform ["LOGIN" port/locals/user-name port/pass] none [ok]
			] [
				auth-done: true
			]
		]
		if not auth-done [
			net-error "No authentication method available"
		]
		path: copy* any [port/path ""]
		if port/target [append path port/target]
		imap-url-parser/do-parse path port/locals/access
		if all [empty? port/locals/access/name port/locals/access/type <> 'list] [
			port/locals/access/name: "INBOX"
		]
		either port/locals/access/section [
			port/locals/send-section: rejoin ["BODY.PEEK[" uppercase port/locals/access/section "]"]
			port/locals/recv-section: rejoin ["BODY[" uppercase port/locals/access/section "]"]
		] [
			port/locals/send-section: port/locals/recv-section: "RFC822"
		]
		switch/all port/locals/access/type [
			list [
				port/locals/list: copy* []
				port/state/tail: 0
				imap-check port reform [uppercase port/locals/access/list {""}
					imap-form-string port/locals/access/name] none [ok]
			]
			box iuid [imap-check port reform ["SELECT" imap-form-string port/locals/access/name] none [ok]]
			iuid [
				port/state/tail: 1
			]
			search [
				port/locals/msg-uids: copy* []
				do select-block
				imap-check port reform ["UID SEARCH" port/locals/access/search] none [ok]
				port/state/tail: length? port/locals/msg-uids
			]
		]
		if all [port/locals/uidvalidity port/locals/access/uidvalidity
		 port/locals/uidvalidity <> port/locals/access/uidvalidity] [
			net-error "Stale IMAP URL"
		]
		port/state/index: 0   ; a zero based index
	]

	imap-read-message: func [
		"Read a message from the IMAP server"
		port
		cmd
		n [integer!]
		/local buf line
	][
		port/locals/msg-num: n
		imap-check port reform [cmd port/locals/msg-num port/locals/send-section] none [ok]
		port/locals/msg
	]

	imap-pick-copy: func [port type /local msgs n] [
		switch port/locals/access/type [
			box [
				either type = 'pick [
					imap-read-message port "FETCH" port/state/index + 1
				] [
					msgs: make block! port/state/num
					repeat n port/state/num [
						append msgs imap-read-message port "FETCH" port/state/index + n
					]
					msgs
				]
			]
			iuid [
				either type = 'pick [
					imap-read-message port "UID FETCH" port/locals/access/uid
				] [
					msgs: make block! 1
					append msgs imap-read-message port "UID FETCH" port/locals/access/uid
					msgs
				]
			]
			list [
				either type = 'pick [
					system/words/pick port/locals/list port/state/index + 1
				] [
					copy/part skip port/locals/list port/state/index n
				]
			]
			search [
				either type = 'pick [
					imap-read-message port "UID FETCH" system/words/pick port/locals/msg-uids port/state/index + 1
				] [
					msgs: make block! port/state/num
					repeat n port/state/num [
						append msgs imap-read-message port "UID FETCH" system/words/pick port/locals/msg-uids port/state/index + n
					]
					msgs
				]
			]
		]
	]
	
	pick: func [
		"Read the Nth message from the POP port"
		port
	][
		imap-pick-copy port 'pick
	]

	copy: func [
		"Copy a set of messages into a block"
		port
	][
		imap-pick-copy port 'copy
	]

	remove: func [
		"Remove the current message"
		port
	][
		switch port/locals/access/type [
			box [
				if port/state/num > 0 [
					imap-check port rejoin ["STORE " port/state/index + 1 ":"
					 port/state/index + port/state/num " +FLAGS.SILENT (\Deleted)"] none [ok]
					imap-check port "EXPUNGE" none [ok]
					port/state/num: 0
				]
			]
			iuid [
				if port/state/num > 0 [
					imap-check port rejoin ["UID STORE " port/locals/access/uid 
					 " +FLAGS.SILENT (\Deleted)"] none [ok]
					imap-check port "EXPUNGE" none [ok]
					port/state/num: 0
				]
			]
			list [
				net-error "Removal of mailboxes not supported"
			]
			search [
				while [port/state/num > 0] [
					imap-check port rejoin ["UID STORE " system/words/pick port/locals/msg-uids port/state/index + 1
					 " +FLAGS.SILENT (\Deleted)"] none [ok]
					system/words/remove at port/locals/msg-uids port/state/index + 1
					port/state/num: port/state/num - 1
				]
				imap-check port "EXPUNGE" none [ok]
			]
		]
		port
	]

	net-utils/net-install IMAP self 143
]
