REBOL [
	Title:  "REBOL Protocols: Root Protocol"
	Author: ["Carl Sassenrath" "Sterling Newton" "Holger Kruse" "Jim Goodnow"]
	Rights: "Copyright REBOL Technologies 1997-2002"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

Root-Protocol: make object! [
	{Common functions and variables used by most protocols.
	These functions ASSUME that the protocol is running in 
	line mode, so CRLF is automatically appended and stripped
	from output and input.  Keep that in mind.}

	port-flags: 0   ; default port flags

	;-- Confirmations (rebound to the confirm function)
	open-check: none
	close-check: none
	write-check: none

	init: func [
		"Parse URL and/or check the port spec object"
		port "Unopened port spec"
		spec "Argument passed to open or make (a URL or port-spec)"
		/local scheme
	][
		if url? spec [net-utils/url-parser/parse-url port spec]
		scheme: port/scheme
		port/url: spec
		if none? port/host [
			net-error reform ["No network server for" scheme "is specified"]
		]
		if none? port/port-id [
			net-error reform ["No port address for" scheme "is specified"]
		]
	]

	open-proto: func [
		"Open the socket connection and confirm server response."
		port "Initalized port spec"
		/sub-protocol subproto
		/secure
		/generic ; ok to use a generic proxy if it is set
		/locals sub-port data in-bypass find-bypass bp
	] [
		if not sub-protocol [subproto: 'tcp]
		net-utils/net-log reduce ["Opening" to-string subproto "for" to-string port/scheme]
		if not system/options/quiet [print ["connecting to:" port/host]]

		find-bypass: func [host bypass /local x] [
			if found? host [
				foreach item bypass [
					if any [
						all [x: find/match/any host item tail? x]
					] [return true]
				]
			]
			false
		]

		in-bypass: func [host bypass /local item x] [
			if any [none? bypass empty? bypass] [return false]
			if not tuple? load host [host: form system/words/read join dns:// host]
			either find-bypass host bypass [
				true
			] [
				host: system/words/read join dns:// host
				find-bypass host bypass
			]
		]

		either all [
			port/proxy/host
			bp: not in-bypass port/host port/proxy/bypass
			find [socks4 socks5 socks] port/proxy/type
		] [ ; using SOCKS proxy
			port/sub-port: net-utils/connect-proxy/sub-protocol port 'connect subproto
		] [
			sub-port: system/words/open/lines compose [     ; if it fails, the port catches it
				scheme: (to-lit-word subproto)
				host: either all [port/proxy/type = 'generic generic bp] [port/proxy/host] [port/proxy/host: none port/host]
				user: port/user ; really go here?
				pass: port/pass ; really go here?
				port-id: either all [port/proxy/type = 'generic generic bp] [port/proxy/port-id] [port/port-id]
			]
			port/sub-port: sub-port
		]
		if all [secure find [ssl tls] subproto] [system/words/set-modes port/sub-port [secure: true]]
		port/sub-port/timeout: port/timeout
		port/sub-port/user: port/user
		port/sub-port/pass: port/pass
		port/sub-port/path: port/path
		port/sub-port/target: port/target
		net-utils/confirm/multiline port/sub-port open-check
		port/state/flags: port/state/flags or port-flags
	]

	open: func [
		"Open the socket connection and confirm server response."
		port "Initalized port spec"
	] [
		open-proto port
	]

	close: func [
		"Quit server, confirm and close the socket connection"
		port "An open port spec"
	][
		port: port/sub-port
		net-utils/confirm port close-check
		system/words/close port

	]

	write: func [    ;(appears as an insert to the user)
		"Default write operation called from buffer layer."
		port "An open port spec"
		data "Data to write"
	][
		net-utils/net-log [ "low level write of " port/state/num "bytes" ]
		write-io port/sub-port data port/state/num
	]

	read: func [
		port "An open port spec"
		data "A buffer to use for the read"
	][
		net-utils/net-log [ "low level read of " port/state/num "bytes" ]
		read-io port/sub-port data port/state/num
	]

	get-sub-port: func [
		port "An open port spec"
	][
		port/sub-port
	]

	awake: func [
		port "An open port spec"
	][
		none
	]

	get-modes: func [
		port "An open port spec"
		modes "A mode block"
	][
		system/words/get-modes port/sub-port modes
	]

	set-modes: func [
		port "An open port spec"
		modes "A mode block"
	][
		system/words/set-modes port/sub-port modes
	]

]
