REBOL [
	Title: "REBOL/View Desktop - Preferences"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

;-- View Preferences Object --------------------------------------------------
;
;	This is the primary structure for holding View preferences.
;	This object is loaded when the user runs View and is saved to
;	disk when the user saves their preferences.
;
;	Because this object is always loaded by the View desktop on
;	startup, it must be loaded safely to avoid a possible virus.
;	The safe way to load the external preferences object is to use
;	the CONSTRUCT function and specify the default object below
;	as the /WITH refinement. [construct/with new-obj user-prefs]

user-prefs: context [
	name: "A Rebol"
	pass: none
	express: none
	server: "none"
	script: "none"
	proxy-user: none
	proxy-pass: none
	root: none
	poll-time: none
	connect-wait: none
	log: off
	debug: off
	window-size: none
	window-offset: none
	icon-offset: none
	keymap: none
	auto-connect: true
	always-close: true
	desktop: true
	skin: none
	desktop-url: http://www.rebol.com/view/desktop/
]

;-- Skin Defaults ------------------------------------------------------------
;
;	These are the default colors and effects that create the primary skin
;	for the desktop.

main-color: 175.155.120
base-color: 160.180.160 ;143.127.111		; background color
button-color: 44.80.132
over-color: button-color	; color when over text
bar-color: 100.120.100 ;180.180.250
base-effect: reduce ['gradient 0x1 base-color + 20 base-color - 40]
bar-effect:  reduce ['gradient 1x0 bar-color + 10 bar-color - 40]

dtw-skin: context [
	main-color: 160.180.160 ;180.200.180 ;140.170.140
	side-color: 100.140.100
	status-color: 80.80.100
	bottom-bar-color:
	top-bar-color: 100.120.100

	main-image: none 
	side-image: none 

	main-effect: [grid 150.170.150] ;170.190.170] ;120.140.120]
	side-effect: [gradient 0x1 200.220.200 100.120.100] ;180.210.180]

	base-text-color: black
	over-text-color: 140.20.20

	info-text-color: rebolor / 3
]

;-- Suffix Mappings ----------------------------------------------------------
;
;	This table maps file suffixes to REBOL words that are used to indicate
;	file types in the launch function. For example, a .txt file will convert
;	to a TEXT word, which will cause the Text Editor to launch.
;
;	This file map can also be loaded from disk as %desktop/filetypes.r.

suffix-map: [
	%.r %.reb %.rip rebol
	%.html %.htm %.xml html
	%.txt %.text text
	%.jpg %.png %.gif %.bmp image
	%.pdf pdf
	%.doc %.rtf %.dot ms-word
	%.xls %.csv ms-excel
]

;-- Desktop Shortcuts --------------------------------------------------------
;
;	This table maps keyboard shortcuts to specific actions.
;	Future versions of ViewTop will allow this to be a preference file.

dtw-keymap: [
	#"^Q" [quit]
	#"^L" [do-local]
	#"^K" [halt-view]
	#"^E" [editor 'same]
	#"^H" [back-folder]
	#"^U" [view-prefs]
	#"^B" [halt]
	#"^R" [reload-folder]
	F5 [reload-folder]
]

ctx-prefs: context [

prefs-file: none

;-- Load and Save Preference File --------------------------------------------

load-prefs: func [
	; Load user preferences from the prefs.r file.
	/local size
][
	if system/user/name [user-prefs/name: system/user/name]

	if all [
		prefs-file
		exists? prefs-file
		attempt [user-prefs: construct/with load prefs-file user-prefs]
	][
		size: system/view/screen-face/size
		if user-prefs/icon-offset [
			user-prefs/icon-offset: max 0x0 min user-prefs/icon-offset size - 100x74
		]

		if user-prefs/window-size [
			user-prefs/window-size: max 150x150 min user-prefs/window-size size
		]

		if user-prefs/window-offset [
			user-prefs/window-offset: max 0x0 min user-prefs/window-offset size - 
				any [user-prefs/window-size 320x320]
		]

		if user-prefs/proxy-user [
			system/schemes/default/proxy/user: user-prefs/proxy-user
		]

		if user-prefs/proxy-pass [
			system/schemes/default/proxy/pass: user-prefs/proxy-pass
		]

		if user-prefs/skin [dtw-skin: construct/with user-prefs/skin dtw-skin]
	]
	user-prefs
]

save-prefs: func [
	; Save user preferences object to the prefs.r file.
	/local str
][
	if not prefs-file [exit]
	str: reform [{REBOL [Title: "User Preferences" Date:} now {]^/}]
	foreach n next first user-prefs [
		repend str [n ": " mold get in user-prefs n newline]
	]
	write prefs-file str
]

;-- Save back to user.r file:
save-user: has [user blk str] [
	;-- User.r (make this obsolete!) (really?)
	if error? try [
		either exists? view-root/user.r [
			; using LOAD/ALL to preserve header
			user: load/all view-root/user.r
			if none? blk: select user 'set-net [repend user ['set-net blk: []]]
			repend clear blk [
				system/user/email
				system/schemes/default/host
				system/schemes/pop/host
				system/schemes/default/proxy/host
				system/schemes/default/proxy/port-id
				system/schemes/default/proxy/type
				; TODO: ESMTP username/password
			]
			remove/part any [find user 'set-user-name ""] 2
			if system/user/name [repend user ['set-user-name system/user/name]]
			save view-root/user.r user
		][write-user]
	][alert "Unable to save settings in your user.r file."]
	;][teller "Unable to save settings in your user.r file." none] ??
]

]